/*
* Copyright (c) 2013, Texas Instruments Incorporated
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
* *  Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
* *  Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
*
* *  Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* --/COPYRIGHT--*/

/*
*  ======== tcpSendReceive.c ========
*  Tool to exercise the TCP portion of the sockets example.
*/

#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <winsock2.h>
#include <ws2tcpip.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include <time.h>

// Need to link with Ws2_32.lib, Mswsock.lib, and Advapi32.lib
#pragma comment (lib, "Ws2_32.lib")
#pragma comment (lib, "Mswsock.lib")
#pragma comment (lib, "AdvApi32.lib")

#include "..\tcptest_config.h"
#include "..\tcptest_osal.h"

#ifdef __cplusplus
extern "C"
{
#endif

	int iINetPollingLoop(SOCKET xServerSocket, uint32_t xFlags);

#ifdef __cplusplus
}
#endif

void vPrintLocalIP(SOCKET listen_sock)
{
	static const char fn[] = "PrntIP";

	sockaddr_in name;
	socklen_t namelen = sizeof(name);
	if (getsockname(listen_sock, (sockaddr*)&name, &namelen) == SOCKET_ERROR)
	{
		printf("%s: Cannot get socket name, error=%d\n", fn, WSAGetLastError());
		return;
	}

	char ac[80];
	if (gethostname(ac, sizeof(ac)) == SOCKET_ERROR) {
		printf("%s: Error %d when getting local host name\n", fn, WSAGetLastError());
		return;
	}

	struct hostent *phe = gethostbyname(ac);
	if (phe == 0) {
		printf("%s: Bad host lookup, error=%d\n", fn, WSAGetLastError());
		return;
	}

	printf("%s: Listening at local machine's IP addresses\n", fn);
	printf("%s:   %s:%d\n", fn, "127.0.0.1", htons(name.sin_port));
	for (int i = 0; phe->h_addr_list[i] != 0; ++i) {
		struct in_addr addr;
		memcpy(&addr, phe->h_addr_list[i], sizeof(struct in_addr));
		printf("%s:   %s:%d\n", fn, inet_ntoa(addr), htons(name.sin_port));
	}
}

int __cdecl main(int argc, char **argv)
{
	static const char fn[] = "main";

	WSADATA wsaData;
	int iResult;

	// Initialize Winsock
	iResult = WSAStartup(MAKEWORD(2, 2), &wsaData);
	if (iResult != 0)
	{
		printf("WSAStartup failed with error: %d\n", iResult);
		return 1;
	}

	struct sockaddr_in server_addr, client_addr;
	SOCKET listen_sock = INVALID_SOCKET;
	SOCKET client_sock = INVALID_SOCKET;
	int len = sizeof(struct sockaddr_in);
	struct timeval timeout = { 10, 0 };

	int listen_timeout = 1000;

	if ((listen_sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == INVALID_SOCKET)
	{
		printf("%s: Cannot create socket, error=%d\n", fn, WSAGetLastError());
		goto QUIT;
	}

	memset(&server_addr, 0, sizeof(struct sockaddr_in));
	server_addr.sin_family = AF_INET;
	server_addr.sin_addr.s_addr = htonl(INADDR_ANY); /* inet_addr("0.0.0.0"); */
	server_addr.sin_port = htons(TCPTEST_TCP_PORT);

	if (bind(listen_sock, (struct sockaddr*)&server_addr, sizeof(struct sockaddr_in)) == SOCKET_ERROR)
	{
		printf("%s: Cannot bind socket, error=%d\n", fn, WSAGetLastError());
		goto QUIT;
	}

	if (listen(listen_sock, 5) == SOCKET_ERROR)
	{
		printf("%s: Cannot listen to socket, error=%d\n", fn, WSAGetLastError());
		goto QUIT;
	}

	/* Set listen timeout to make watchdog work */
	timeout.tv_sec = listen_timeout;
	if (setsockopt(listen_sock, SOL_SOCKET, SO_RCVTIMEO, (const char*)&timeout, sizeof(timeout)) == SOCKET_ERROR)
	{
		printf("%s: Cannot set socket option SO_RCVTIMEO, error=%d\n", fn, WSAGetLastError());
		goto QUIT;
	}

	vPrintLocalIP(listen_sock);

	for (;;)
	{
		printf("%s: Waiting for client to connect..\n", fn);
		if ((client_sock = accept(listen_sock, (struct sockaddr*)&client_addr, &len)) == INVALID_SOCKET)
		{
			/* Timeout, Wait a short time and try again */
			printf("%s: Cannot accept client connection, error=%d\n", fn, WSAGetLastError());
			Sleep(100);
			break;
		}

		iINetPollingLoop(client_sock, TCPTEST_RECV_FLAGS);

		if (client_sock != INVALID_SOCKET)
		{
			closesocket(client_sock);
		}
	}

QUIT:

	if (listen_sock != INVALID_SOCKET)
	{
		closesocket(listen_sock);
	}

	// release Winsock
	WSACleanup();

	return 0;
}
