/*
* Copyright (c) 2013, Texas Instruments Incorporated
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
* *  Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
* *  Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
*
* *  Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* --/COPYRIGHT--*/

/*
*  ======== tcpSendReceive.c ========
*  Tool to exercise the TCP portion of the sockets example.
*/

#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <winsock2.h>
#include <ws2tcpip.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "../tcptest_config.h"

// Need to link with Ws2_32.lib, Mswsock.lib, and Advapi32.lib
#pragma comment (lib, "Ws2_32.lib")
#pragma comment (lib, "Mswsock.lib")
#pragma comment (lib, "AdvApi32.lib")

#define MAXBUF    1024

char getRandomCharacter()
{
	char randomNumber = 0;

	randomNumber = rand() % CHAR_MAX;

	return randomNumber;
}

void randomArrayFill(char* arrayFill, int size)
{
	srand(time(0)); //Seed the random system

	for (int i = 0; i < size; i++)
	{
		arrayFill[i] = getRandomCharacter();
	}
}

int __cdecl main(int argc, char **argv)
{
	WSADATA wsaData;
	int iResult;

	int i;
	int sockfd = 0;
	int bytes_read, total_bytes_read, bytes_sent, total_bytes_sent;
	struct addrinfo hints;
	struct addrinfo *results = NULL;
	int status = EXIT_SUCCESS;
	int count = 0;
	int id;
	int value;
	unsigned int sleepTime = 1000;
	unsigned int buffSize = MAXBUF;
	char *buffer = NULL;
	time_t start;
	start = time(NULL);

	/* parameter check */
	if (argc < 4 || argc > 6) 
	{
		printf("usage: %s <IPv4 or IPv6 addr> <port> <id> -l[length] -s[sleep in mS]\n", argv[0]);
		status = EXIT_FAILURE;
		goto QUIT;
	}

	id = atoi(argv[3]);

	/* Parse options */
	i = argc - 1;
	while ((i > 3) && (argv[i][0] == '-')) 
	{
		switch (argv[i][1]) 
		{
		case 'l':
			buffSize = atoi(&argv[i][2]);
			break;
		case 's':
			sleepTime = atoi(&argv[i][2]);
			break;
		default:
			printf("Valid options are -l[length] and -s[sleep in mS]\n");
			status = EXIT_FAILURE;
			goto QUIT;
		}
		i--;
	}

	buffer = (char*)malloc(2 * buffSize);
	if (buffer == NULL) 
	{
		printf("malloc failed\n");
		status = EXIT_FAILURE;
	}
	memset(buffer, 0, 2 * buffSize);

	// Initialize Winsock
	iResult = WSAStartup(MAKEWORD(2, 2), &wsaData);
	if (iResult != 0) 
	{
		printf("WSAStartup failed with error: %d\n", iResult);
		return 1;
	}

	memset(&hints, 0, sizeof(hints));
	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;
	hints.ai_protocol = IPPROTO_TCP;

	/*
	* getaddrinfo() fills in the results struct for us appropriately
	* depending on whether the IP address is v4 or v6
	*
	* argv[1] = IPv4 or IPv6 address passed in from command line
	* argv[2] = port number passed in from command line
	*/
	value = getaddrinfo(argv[1], argv[2], &hints, &results);

	if (value != 0) 
	{
		fprintf(stderr, "getaddrinfo failed: %d\n", value);
		if (value == -2 || value == 11004) 
		{
			fprintf(stderr, "unrecognized IP address\n");
		}
		status = EXIT_FAILURE;
		goto QUIT;
	}

	/* create socket. ai_family determined for us via getaddrinfo() call */
	if ((sockfd = socket(results->ai_family, SOCK_STREAM, 0)) < 0) {
		fprintf(stderr, "socket failed: %d\n", errno);
		status = errno;
		goto QUIT;
	}

	/* connect. ai_addr set to AF_INET or AF_INET6 by getaddrinfo() call */
	if (connect(sockfd, results->ai_addr, results->ai_addrlen) < 0) 
	{
		printf("connect failed: %d\n", errno);
		status = errno;
		goto QUIT;
	}

	printf("Starting test, data size = %d, transmit delay = %d uSec\n", buffSize, sleepTime);

	/* loop */
	total_bytes_sent = 0;
	i = 0;
	while (1) 
	{
		randomArrayFill(buffer, buffSize);

		bytes_sent = send(sockfd, buffer, buffSize, 0);
		if (bytes_sent != buffSize) 
		{
			printf("[id %d] stopping test, retval=%d, error=%d, bytes_sent=%d\n", 
				id, bytes_sent, WSAGetLastError(), total_bytes_sent);
			status = EXIT_FAILURE;
			goto QUIT;
		}

		total_bytes_sent += bytes_sent;

		if (total_bytes_sent > TCPTEST_BYTES_READ_TRESHOLD)
		{
			printf("[id %d] sent = %d, time = %ld\n", id, total_bytes_sent, time(NULL) - start);
			total_bytes_sent = 0;
		}

		/* Sleep specified time */
		Sleep(sleepTime);
	}

QUIT:
	/* clean up */
	if (sockfd) 
	{
		closesocket(sockfd);
	}

	if (results) 
	{
		freeaddrinfo(results);
	}

	// release Winsock
	WSACleanup();

	if (buffer) 
	{
		free(buffer);
	}

	return (status);
}
